@@ -24,7 +24,7 @@
 #define HANDLE_SIZE 8
 #define MIN_WIDTH 200
 #define XOR_RECT_WIDTH 2
-#define UNSELECT_TIMEOUT 150 /* ms */
+#define UNSELECT_TIMEOUT 0 /* ms */
 
 /* Size of the pixmaps */
 #define DECOR_WIDTH      16
@@ -35,6 +35,7 @@ typedef struct {
 	GtkWidget  *widget;
 	GdkWindow  *window;
 	GdkWindow  *decor_window;
+	guint       focus_out_id;
 	int         lower_row; /* zero is first displayed row */
 	int         rows_used;
 	int         x;         /* coords of child's window */
@@ -109,7 +110,6 @@ static void gncal_full_day_forall         (GtkContainer      *container,
 
 static void range_activated (GncalFullDay *fullday);
 
-
 static GtkContainerClass *parent_class;
 
 static int fullday_signals[LAST_SIGNAL] = { 0 };
@@ -277,6 +277,9 @@ child_realize (GncalFullDay *fullday, Child *child)
 static void
 child_unrealize (GncalFullDay *fullday, Child *child)
 {
+	if (GTK_WIDGET_REALIZED (child->widget))
+		gtk_widget_unrealize (child->widget);
+
 	gdk_window_set_user_data (child->window, NULL);
 	gdk_window_destroy (child->window);
 	child->window = NULL;
@@ -422,6 +425,22 @@ edit_appointment (GtkWidget *widget, gpointer data)
 	gtk_widget_show (ee);
 }
 
+static void
+delete_occurance (GtkWidget *widget, gpointer data)
+{
+	Child *child;
+	iCalObject *ical;
+	time_t *t;
+
+	child = data;
+	ical = child->ico;
+	t = g_new(time_t, 1);
+	*t = child->start;
+	ical->exdate = g_list_prepend(ical->exdate, t);
+	gnome_calendar_object_changed(GNCAL_FULL_DAY (child->widget->parent)->calendar, child->ico, CHANGE_DATES);
+	
+}
+
 static void
 delete_appointment (GtkWidget *widget, gpointer data)
 {
@@ -462,35 +481,47 @@ unrecur_appointment (GtkWidget *widget, gpointer data)
 static void
 child_popup_menu (GncalFullDay *fullday, Child *child, GdkEventButton *event)
 {
-	int sensitive, idx, items;
+	int sensitive, items;
+	struct menu_item *context_menu;
 	
 	static struct menu_item child_items[] = {
-		{ N_("Make this appointment movable"), (GtkSignalFunc) unrecur_appointment, NULL, TRUE },
 		{ N_("Edit this appointment..."), (GtkSignalFunc) edit_appointment, NULL, TRUE },
 		{ N_("Delete this appointment"), (GtkSignalFunc) delete_appointment, NULL, TRUE },
 		{ NULL, NULL, NULL, TRUE },
 		{ N_("New appointment..."), (GtkSignalFunc) new_appointment, NULL, TRUE }
 	};
 
-	child_items[0].data = child;
-	child_items[1].data = child;
-	child_items[2].data = child;
-	child_items[4].data = fullday;
+	static struct menu_item recur_child_items[] = {
+		{ N_("Make this appointment movable"), (GtkSignalFunc) unrecur_appointment, NULL, TRUE },
+		{ N_("Edit this appointment..."), (GtkSignalFunc) edit_appointment, NULL, TRUE },
+		{ N_("Delete this occurance"), (GtkSignalFunc) delete_occurance, NULL, TRUE },
+		{ N_("Delete all occurances"), (GtkSignalFunc) delete_appointment, NULL, TRUE },
+		{ NULL, NULL, NULL, TRUE },
+		{ N_("New appointment..."), (GtkSignalFunc) new_appointment, NULL, TRUE }
+	};
 
 	sensitive = (child->ico->user_data == NULL);
 
-	child_items[0].sensitive = sensitive;
-	child_items[1].sensitive = sensitive;
-	child_items[2].sensitive = sensitive;
-
 	if (child->ico->recur){
-		idx   = 0;
-		items = 5;
+		items = 6;
+		context_menu = &recur_child_items[0];
+		context_menu[2].data = child;
+		context_menu[3].data = child;
+		context_menu[4].data = fullday;
+		context_menu[3].sensitive = sensitive;
 	} else {
-		idx   = 1;
 		items = 4;
+		context_menu = &child_items[0];
+		context_menu[3].data = fullday;
 	}
-	popup_menu (&child_items [idx], items, event);
+	/* These settings are common for each context sensitive menu */
+	context_menu[0].data = child;
+	context_menu[1].data = child;
+	context_menu[0].sensitive = sensitive;
+	context_menu[1].sensitive = sensitive;
+	context_menu[2].sensitive = sensitive;
+
+	popup_menu (context_menu, items, event);
 }
 
 static void
@@ -503,10 +534,6 @@ child_realized_setup (GtkWidget *widget, gpointer data)
 	fullday = GNCAL_FULL_DAY (widget->parent);
 
 	gdk_window_set_cursor (widget->window, fullday->beam_cursor);
-
-	gtk_text_insert (GTK_TEXT (widget), NULL, NULL, NULL,
-			 child->ico->summary,
-			 strlen (child->ico->summary));
 }
 
 static void
@@ -571,13 +598,7 @@ child_set_size (Child *child)
 static gint
 child_focus_in (GtkWidget *widget, GdkEventFocus *event, gpointer data)
 {
-	Child *child;
-
-	child = data;
-
-	child_set_size (child);
-	gdk_window_raise (child->window);
-
+	child_set_size (data);
 	return FALSE;
 }
 
@@ -587,7 +608,7 @@ child_focus_out (GtkWidget *widget, GdkEventFocus *event, gpointer data)
 	Child *child;
 	GncalFullDay *fullday;
 	char *text;
-	
+
 	child = data;
 
 	child_set_size (child);
@@ -639,6 +660,7 @@ child_button_press (GtkWidget *widget, GdkEventButton *event, gpointer data)
 	fullday = GNCAL_FULL_DAY (widget->parent);
 
 	gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "button_press_event");
+	gtk_widget_grab_focus (widget);
 	child_popup_menu (fullday, child, event);
 
 	return TRUE;
@@ -688,7 +710,12 @@ child_new (GncalFullDay *fullday, time_t start, time_t end, iCalObject *ico)
 	child_range_changed (fullday, child);
 	child_compute_decor (child);
 
-	/* We set the i-beam cursor and the initial summary text upon realization */
+	if (ico->summary)
+		gtk_text_insert (GTK_TEXT (child->widget), NULL, NULL, NULL,
+				 ico->summary,
+				 strlen (ico->summary));
+
+	/* We set the i-beam cursor of the text widget upon realization */
 
 	gtk_signal_connect (GTK_OBJECT (child->widget), "realize",
 			    (GtkSignalFunc) child_realized_setup,
@@ -698,9 +725,9 @@ child_new (GncalFullDay *fullday, time_t start, time_t end, iCalObject *ico)
 				  (GtkSignalFunc) child_focus_in,
 				  child);
 
-	gtk_signal_connect_after (GTK_OBJECT (child->widget), "focus_out_event",
-				  (GtkSignalFunc) child_focus_out,
-				  child);
+	child->focus_out_id = gtk_signal_connect_after (GTK_OBJECT (child->widget), "focus_out_event",
+							(GtkSignalFunc) child_focus_out,
+							child);
 
 	gtk_signal_connect (GTK_OBJECT (child->widget), "key_press_event",
 			    (GtkSignalFunc) child_key_press,
@@ -720,14 +747,18 @@ child_new (GncalFullDay *fullday, time_t start, time_t end, iCalObject *ico)
 	return child;
 }
 
+static void
+squick (GtkWidget *widget, gpointer data)
+{
+}
+
 static void
 child_destroy (GncalFullDay *fullday, Child *child)
 {
-	/* Unparent the child widget manually as we don't have a remove method */
-
-	gtk_widget_ref (child->widget);
-
-	gtk_widget_unparent (child->widget);
+	/* Disconnect the focus_out_event signal since we will get such an event
+	 * from the destroy call.
+	 */
+	gtk_signal_disconnect (GTK_OBJECT (child->widget), child->focus_out_id);
 
 	if (GTK_WIDGET_MAPPED (fullday))
 		child_unmap (fullday, child);
@@ -735,8 +766,13 @@ child_destroy (GncalFullDay *fullday, Child *child)
 	if (GTK_WIDGET_REALIZED (fullday))
 		child_unrealize (fullday, child);
 
-	gtk_widget_unref (child->widget);
+	/* Unparent the child widget manually as we don't have a remove method */
 
+	gtk_signal_connect (GTK_OBJECT (child->widget), "destroy",
+			    (GtkSignalFunc) squick,
+			    NULL);
+
+	gtk_widget_unparent (child->widget);
 	g_free (child);
 }
 
@@ -923,28 +959,30 @@ gncal_full_day_init (GncalFullDay *fullday)
 	fullday->bell_gc = NULL;
 }
 
+/* Destroys all the children in the full day widget */
+static void
+destroy_children (GncalFullDay *fullday)
+{
+	GList *children;
+
+	for (children = fullday->children; children; children = children->next)
+		child_destroy (fullday, children->data);
+
+	g_list_free (fullday->children);
+	fullday->children = NULL;
+}
+
 static void
 gncal_full_day_destroy (GtkObject *object)
 {
 	GncalFullDay *fullday;
-	GList *children;
-	Child *child;
 
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (GNCAL_IS_FULL_DAY (object));
 
 	fullday = GNCAL_FULL_DAY (object);
 
-	/* Unparent the children manually as we don't have a remove method */
-
-	for (children = fullday->children; children; children = children->next) {
-		child = children->data;
-
-		gtk_widget_unparent (child->widget);
-	}
-
-	g_list_free (fullday->children);
-	fullday->children = NULL;
+	destroy_children (fullday);
 
 	g_free (fullday->drag_info);
 
@@ -1816,10 +1854,10 @@ gncal_full_day_button_release (GtkWidget *widget, GdkEventButton *event)
 		break;
 
 	case DRAG_SELECT:
-		if ((event->time - di->click_time) < UNSELECT_TIMEOUT)
-			di->sel_rows_used = 0;
-		else
-			recompute_motion (fullday, y);
+	    if ((event->time - di->click_time) < UNSELECT_TIMEOUT)
+		    di->sel_rows_used = 0;
+	    else
+		    recompute_motion (fullday, y);
 
 		gdk_pointer_ungrab (event->time);
 
@@ -1960,26 +1998,34 @@ gncal_full_day_key_press (GtkWidget *widget, GdkEventKey *event)
 		return TRUE;
 	}
 
-	if (event->length > 0) {
-		/* This means some printable key was pressed */
+	/*
+	 * If a non-printable key was pressed, bail.  Otherwise, begin
+	 * editing the appointment.
+	 */
+	if ((event->keyval < 0x20) || (event->keyval > 0xFF)
+	    || (event->length == 0) || (event->state & GDK_CONTROL_MASK)
+	    || (event->state & GDK_MOD1_MASK))
+		return FALSE;
 
-		gtk_signal_emit (GTK_OBJECT (fullday), fullday_signals[RANGE_ACTIVATED]);
+	gtk_signal_emit (GTK_OBJECT (fullday),
+			 fullday_signals[RANGE_ACTIVATED]);
+	
+	/*
+	 * Find the new child, which should hopefully be focused, and
+	 * insert the keypress.
+	 */
+	for (children = fullday->children; children; children = children->next) {
+		child = children->data;
 
-		/* Find the new child, which should hopefully be focused, and insert the keypress */
+		if (GTK_WIDGET_HAS_FOCUS (child->widget)) {
+			pos = gtk_text_get_length (GTK_TEXT (child->widget));
 
-		for (children = fullday->children; children; children = children->next) {
-			child = children->data;
+			gtk_editable_insert_text (GTK_EDITABLE (child->widget),
+						  event->string,
+						  event->length,
+						  &pos);
 
-			if (GTK_WIDGET_HAS_FOCUS (child->widget)) {
-				pos = gtk_text_get_length (GTK_TEXT (child->widget));
-
-				gtk_editable_insert_text (GTK_EDITABLE (child->widget),
-							  event->string,
-							  event->length,
-							  &pos);
-
-				return TRUE;
-			}
+			return TRUE;
 		}
 	}
 
@@ -2089,19 +2135,14 @@ gncal_full_day_update (GncalFullDay *fullday, iCalObject *ico, int flags)
 
 	/* We have to regenerate and layout our list of children */
 
-	for (children = fullday->children; children; children = children->next)
-		child_destroy (fullday, children->data);
+	destroy_children (fullday);
 
-	g_list_free (fullday->children);
-
-	fullday->children = NULL;
-	
 	calendar_iterate (fullday->calendar->cal,
 			  fullday->lower,
 			  fullday->upper,
 			  fullday_add_children,
 			  fullday);
-	
+
 	layout_children (fullday);
 
 	/* Realize and map children */
@@ -2178,7 +2219,6 @@ gncal_full_day_focus_child (GncalFullDay *fullday, iCalObject *ico)
 {
 	GList *children;
 	Child *child;
-	GdkEvent event;
 
 	g_return_if_fail (fullday != NULL);
 	g_return_if_fail (ico != NULL);
@@ -2188,22 +2228,6 @@ gncal_full_day_focus_child (GncalFullDay *fullday, iCalObject *ico)
 
 		if (child->ico == ico) {
 			gtk_widget_grab_focus (child->widget);
-
-			/* We synthesize a click because GtkText will not set the cursor and
-			 * the user will not be able to type-- this has to be fixed in
-			 * GtkText.  */
-
-			memset (&event, 0, sizeof (event));
-
-			event.type = GDK_BUTTON_PRESS;
-			event.button.window = child->widget->window;
-			event.button.time = GDK_CURRENT_TIME;
-			event.button.x = 0;
-			event.button.y = 0;
-			event.button.button = 1;
-
-			gtk_widget_event (child->widget, &event);
-
 			break;
 		}
 	}
