@@ -11,6 +11,7 @@
 #include <gnome.h>
 #include <pwd.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <string.h>
 #include <ctype.h>
 #include "calendar.h"
@@ -42,6 +43,9 @@ int day_begin, day_end;
 /* Whether weeks starts on Sunday or Monday */
 int week_starts_on_monday;
 
+/* If true, do not show our top level window */
+int startup_hidden = 0;
+
 /* The array of color properties -- keep in sync with the enumeration defined in main.h.  The color
  * values specified here are the defaults for the program.
  */
@@ -61,7 +65,7 @@ int active_calendars = 0;
 /* A list of all of the calendars started */
 GList *all_calendars = NULL;
 
-static void new_calendar (char *full_name, char *calendar_file, char *geometry, char *view);
+static void new_calendar (char *full_name, char *calendar_file, char *geometry, char *view, gboolean hidden);
 
 /* For dumping part of a calendar */
 static time_t from_t, to_t;
@@ -304,14 +308,14 @@ goto_clicked (GtkWidget *widget, GnomeCalendar *gcal)
 static void
 new_calendar_cmd (GtkWidget *widget, void *data)
 {
-	new_calendar (full_name, NULL, NULL, NULL);
+	new_calendar (full_name, NULL, NULL, NULL, FALSE);
 }
 
 static void
 open_ok (GtkWidget *widget, GtkFileSelection *fs)
 {
 	/* FIXME: find out who owns this calendar and use that name */
-	new_calendar ("Somebody", gtk_file_selection_get_filename (fs), NULL, NULL);
+	new_calendar ("Somebody", gtk_file_selection_get_filename (fs), NULL, NULL, FALSE);
 
 	gtk_widget_destroy (GTK_WIDGET (fs));
 }
@@ -391,9 +395,34 @@ save_calendar_cmd (GtkWidget *widget, void *data)
 {
 	GnomeCalendar *gcal = data;
 
-	if (gcal->cal->filename)
+	if (gcal->cal->filename){
+		struct stat s;
+		
+		stat (gcal->cal->filename, &s);
+
+		if (s.st_mtime != gcal->cal->file_time){
+			GtkWidget *box;
+			char *str;
+			int b;
+			
+			str = g_strdup_printf (
+				_("File %s has changed since it was loaded\nContinue?"),
+				gcal->cal->filename);
+			box = gnome_message_box_new (str, GNOME_MESSAGE_BOX_INFO,
+						     GNOME_STOCK_BUTTON_YES,
+						     GNOME_STOCK_BUTTON_NO,
+						     NULL);
+			g_free (str);
+			gnome_dialog_set_default (GNOME_DIALOG (box), 1);
+			b = gnome_dialog_run (GNOME_DIALOG (box));
+			gtk_object_destroy (GTK_OBJECT (box));
+
+			if (b != 0)
+				return;
+		}
+		
 		calendar_save (gcal->cal, gcal->cal->filename);
-	else
+	} else
 		save_as_calendar_cmd (widget, data);
 }
 
@@ -479,8 +508,8 @@ setup_appbar (GtkWidget *gcal)
 {
         GnomeAppBar *appbar;
 	
-        appbar = gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_USER);
-	gnome_app_set_statusbar(GNOME_APP (gcal), GTK_WIDGET(appbar));
+        appbar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_USER);
+	gnome_app_set_statusbar (GNOME_APP (gcal), GTK_WIDGET(appbar));
 }
 
 static gint
@@ -491,7 +520,7 @@ calendar_close_event (GtkWidget *widget, GdkEvent *event, GnomeCalendar *gcal)
 }
 
 static void
-new_calendar (char *full_name, char *calendar_file, char *geometry, char *page)
+new_calendar (char *full_name, char *calendar_file, char *geometry, char *page, gboolean hidden)
 {
 	GtkWidget   *toplevel;
 	char        title[128];
@@ -502,6 +531,7 @@ new_calendar (char *full_name, char *calendar_file, char *geometry, char *page)
 	g_snprintf(title, 128, _("%s%s"), full_name, _("'s calendar"));
 
 	toplevel = gnome_calendar_new (title);
+	    
 	if (gnome_parse_geometry (geometry, &xpos, &ypos, &width, &height)){
 		if (xpos != -1)
 			gtk_widget_set_uposition (toplevel, xpos, ypos);
@@ -525,6 +555,17 @@ new_calendar (char *full_name, char *calendar_file, char *geometry, char *page)
 
 	active_calendars++;
 	all_calendars = g_list_prepend (all_calendars, toplevel);
+	gtk_widget_realize (toplevel);
+
+	if (hidden){
+		GnomeWinState state;
+		
+		state = gnome_win_hints_get_state (toplevel);
+
+		state |= WIN_STATE_MINIMIZED;
+		gnome_win_hints_set_state (toplevel, state);
+	}
+	
 	gtk_widget_show (toplevel);
 }
 
@@ -541,7 +582,8 @@ process_dates (void)
 enum {
 	GEOMETRY_KEY = -1,
 	USERFILE_KEY = -2,
-	VIEW_KEY     = -3
+	VIEW_KEY     = -3,
+	HIDDEN_KEY   = -4,
 };
 
 /* Lists used to startup various GnomeCalendars */
@@ -645,21 +687,26 @@ parse_an_arg (poptContext ctx,
 		show_events = 1;
 		break;
 
+	case HIDDEN_KEY:
+		startup_hidden = 1;
+		break;
+		
 	default:
 	}
 }
 
-static const struct poptOption options[] = {
-  {NULL, '\0', POPT_ARG_CALLBACK, parse_an_arg, 0, NULL, NULL},
-  {"events", 'e', POPT_ARG_NONE, NULL, 'e', N_("Show events and quit"),
-   NULL},
-  {"from", 'f', POPT_ARG_STRING, NULL, 'f', N_("Specifies start date [for --events]"), N_("DATE")},
-  {"file", 'F', POPT_ARG_STRING, NULL, 'F', N_("File to load calendar from"), N_("FILE")},
-  {"userfile", '\0', POPT_ARG_NONE, NULL, USERFILE_KEY, N_("Load the user calendar"), NULL},
-  {"geometry", '\0', POPT_ARG_STRING, NULL, GEOMETRY_KEY, N_("Geometry for starting up"), N_("GEOMETRY")},
-  {"view", '\0', POPT_ARG_STRING, NULL, VIEW_KEY, N_("The startup view mode"), N_("VIEW")},
-  {"to", 't', POPT_ARG_STRING, NULL, 't', N_("Specifies ending date [for --events]"), N_("DATE")},
-  {NULL, '\0', 0, NULL, 0}
+static const struct poptOption options [] = {
+	{ NULL, '\0', POPT_ARG_CALLBACK, parse_an_arg, 0, NULL, NULL },
+	{ "events", 'e', POPT_ARG_NONE, NULL, 'e', N_("Show events and quit"),
+	  NULL },
+	{ "from", 'f', POPT_ARG_STRING, NULL, 'f', N_("Specifies start date [for --events]"), N_("DATE") },
+	{ "file", 'F', POPT_ARG_STRING, NULL, 'F', N_("File to load calendar from"), N_("FILE") },
+	{ "userfile", '\0', POPT_ARG_NONE, NULL, USERFILE_KEY, N_("Load the user calendar"), NULL },
+	{ "geometry", '\0', POPT_ARG_STRING, NULL, GEOMETRY_KEY, N_("Geometry for starting up"), N_("GEOMETRY") },
+	{ "view", '\0', POPT_ARG_STRING, NULL, VIEW_KEY, N_("The startup view mode"), N_("VIEW") },
+	{ "to", 't', POPT_ARG_STRING, NULL, 't', N_("Specifies ending date [for --events]"), N_("DATE") },
+	{ "hidden", 0, POPT_ARG_NONE, NULL, HIDDEN_KEY, N_("If used, starts in iconic mode"), NULL },
+	{ NULL, '\0', 0, NULL, 0}
 };
 
 static void
@@ -718,14 +765,14 @@ main(int argc, char *argv[])
 {
 	GnomeClient *client;
 
-	bindtextdomain(PACKAGE, GNOMELOCALEDIR);
-	textdomain(PACKAGE);
+	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
+	textdomain (PACKAGE);
 
-	gnome_init_with_popt_table("calendar", VERSION, argc, argv,
-				   options, 0, NULL);
+	gnome_init_with_popt_table ("calendar", VERSION, argc, argv,
+				    options, 0, NULL);
 
 	if (show_events)
-	  dump_events ();
+		dump_events ();
 
 	client = gnome_master_client ();
 	if (client){
@@ -762,7 +809,7 @@ main(int argc, char *argv[])
 				title = full_name;
 			else
 				title = file;
-			new_calendar (title, file, geometry, page_name);
+			new_calendar (title, file, geometry, page_name, startup_hidden);
 
 			p = p->next;
 			if (g)
@@ -775,7 +822,7 @@ main(int argc, char *argv[])
 		char *geometry = start_geometries ? start_geometries->data : NULL;
 		char *page_name = start_views ? start_views->data : NULL;
 
-		new_calendar (full_name, user_calendar_file, geometry, page_name);
+		new_calendar (full_name, user_calendar_file, geometry, page_name, startup_hidden);
 	}
 	gtk_main ();
 	return 0;
