@@ -1,3 +1,24 @@
+/* GnomeCard - a graphical contact manager.
+ *
+ * canvas.c: This file is part of GnomeCard.
+ * 
+ * Copyright (C) 1999 The Free Software Foundation
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
+
 #include <config.h>
 #include <gnome.h>
 
@@ -9,6 +30,7 @@
 #include "canvas.h"
 #include "dnd.h"
 
+#define CARDNAME_FONT "-adobe-helvetica-medium-r-normal-*-*-180-*-*-p-*-iso8859-1"
 #define CANVAS_FONT "-adobe-helvetica-medium-r-*-*-*-120-*-*-p-*-iso8859-1"
 #define ADDR_FONT "-adobe-helvetica-medium-r-*-*-*-100-*-*-p-*-iso8859-1"
 #define CANVAS_WIDTH 225
@@ -33,6 +55,7 @@ typedef struct {
 
 GtkWidget *gnomecard_canvas;
 static GnomeCanvasItem *cardname, *fullname, *workphone, *homephone;
+static GnomeCanvasItem *cardname_box, *phone_box, *postal_box, *memo_box;
 static GnomeCanvasItem *email, *url, *org, *title, *memos;
 static CanvasAddressItem homeaddr, workaddr;
 static GnomeCanvasGroup *root;
@@ -65,7 +88,7 @@ gnomecard_canvas_new(void)
 	root = GNOME_CANVAS_GROUP (gnome_canvas_root(GNOME_CANVAS(canvas)));
 
 	/* label at top of canvas, contains current cardname */
-	gnome_canvas_item_new (root, gnome_canvas_rect_get_type (),
+	cardname_box = gnome_canvas_item_new (root, gnome_canvas_rect_get_type (),
 			       "x1", 5.0,
 			       "y1", 5.0,
 			       "x2", (double) (CANVAS_WIDTH - 6.0),
@@ -77,10 +100,10 @@ gnomecard_canvas_new(void)
 
 	cardname = gnome_canvas_item_new (root, gnome_canvas_text_get_type (),
 				      "text", "",
-				      "x", CANVAS_WIDTH / 2.0,
+				      "x", 6.0, /*CANVAS_WIDTH / 2.0,*/
 				      "y", 20.0,
-				      "font", "-adobe-helvetica-medium-r-normal-*-*-180-*-*-p-*-iso8859-1",
-				      "anchor", GTK_ANCHOR_CENTER,
+				      "font", CARDNAME_FONT,
+				      "anchor", GTK_ANCHOR_WEST,
 				      "fill_color", HEADER_TEXT_COLOR,
 				      NULL);
 
@@ -138,7 +161,7 @@ gnomecard_canvas_new(void)
 
 	/* phone #'s */
 	ypos += LIST_SPACING;
-	gnome_canvas_item_new (root, gnome_canvas_rect_get_type (),
+	phone_box = gnome_canvas_item_new (root, gnome_canvas_rect_get_type (),
 			       "x1", 5.0,
 			       "y1", ypos,
 			       "x2", (double) (CANVAS_WIDTH - 6.0),
@@ -216,7 +239,7 @@ gnomecard_canvas_new(void)
 
 	/* addresses */
 	ypos += LIST_SPACING;
-	gnome_canvas_item_new (root, gnome_canvas_rect_get_type (),
+	postal_box = gnome_canvas_item_new (root, gnome_canvas_rect_get_type (),
 			       "x1", 5.0,
 			       "y1", ypos,
 			       "x2", (double) (CANVAS_WIDTH - 6.0),
@@ -370,7 +393,7 @@ gnomecard_canvas_new(void)
 
 	/* memo(s) */
 	ypos += LIST_SPACING;
-	gnome_canvas_item_new (root, gnome_canvas_rect_get_type (),
+	memo_box = gnome_canvas_item_new (root, gnome_canvas_rect_get_type (),
 			       "x1", 5.0,
 			       "y1", ypos,
 			       "x2", (double) (CANVAS_WIDTH - 6.0),
@@ -449,141 +472,169 @@ void gnomecard_clear_canvas(void)
 void
 gnomecard_update_canvas(Card *crd) 
 {
-    GList *l;
-    gchar *name;
-    gdouble x1, x2, y1, y2;
-
-    if (!crd) {
-	g_message("NULL card in gnomecard_update_canvas!");
-	return;
-    }
-
-    /* card name */
-    if (crd->fname.str)
-	gnomecard_canvas_text_item_set(cardname, crd->fname.str);
-    else
-	gnomecard_canvas_text_item_set(cardname, _("No fname for this card."));
-
-    name = gnomecard_join_name(crd->name.prefix,
-			       crd->name.given, 
-			       crd->name.additional,
-			       crd->name.family, 
-			       crd->name.suffix);
-    /* full name */
-    gnomecard_canvas_text_item_set(fullname, name);
-
-    /* title and organization */
-    if (crd->title.str)
-	gnomecard_canvas_text_item_set(title, crd->title.str);
-    else
-	gnomecard_canvas_text_item_set(title, "");
-
-    gnomecard_canvas_text_item_set(org, crd->org.name);
-
-    /* email and web site */
-    gnomecard_canvas_text_item_set(email, crd->email.address);
-
-    if (crd->url.str)
-	gnomecard_canvas_text_item_set(url, crd->url.str);
-    else
-	gnomecard_canvas_text_item_set(url, "");
-
-    /* phone numbers */
-    if (crd->phone.l) {
 	GList *l;
+	gchar *name;
+	gdouble x1, x2, y1, y2;
 	
-	l = findmatchPhoneType(crd->phone.l, PHONE_HOME);
-	if (l) {
+	g_assert(crd != NULL);
+	
+	/* card name */
+	{
+		GdkFont *font;
+		double width;
+		char *str;
+		
+		str = (crd->fname.str)? crd->fname.str : _("No fname for this card.");
+		
+		font = gdk_font_load(CARDNAME_FONT);
+		width = gdk_string_width(font, str);
+		gdk_font_unref(font);
+		
+		width = MAX(CANVAS_WIDTH - 6.0, width + 7.0);
+		
+		gnomecard_canvas_text_item_set(cardname, str);
+		gnome_canvas_item_set(cardname_box, "x2", width, NULL);
+		gnome_canvas_item_set(phone_box, "x2", width, NULL);
+		gnome_canvas_item_set(postal_box, "x2", width, NULL);
+		gnome_canvas_item_set(memo_box, "x2", width, NULL);
+	}
+	
+	name = gnomecard_join_name(crd->name.prefix,
+														 crd->name.given, 
+														 crd->name.additional,
+														 crd->name.family, 
+														 crd->name.suffix);
+	/* full name */
+	gnomecard_canvas_text_item_set(fullname, name);
+	
+	/* title and organization */
+	if (crd->title.str)
+		gnomecard_canvas_text_item_set(title, crd->title.str);
+	else
+		gnomecard_canvas_text_item_set(title, "");
+	
+	gnomecard_canvas_text_item_set(org, crd->org.name);
+	
+	/* email and web site */
+	gnomecard_canvas_text_item_set(email, crd->email.address);
+
+	if (crd->url.str)
+		gnomecard_canvas_text_item_set(url, crd->url.str);
+	else
+		gnomecard_canvas_text_item_set(url, "");
+	
+	/* phone numbers */
+	if (crd->phone.l) {
+		GList *l;
+		
+		l = findmatchPhoneType(crd->phone.l, PHONE_HOME);
+		if (l) {
 	    CardPhone *phone;
 	    
 	    phone = (CardPhone *)l->data;
 	    gnomecard_canvas_text_item_set(homephone, phone->data);
-	} else {
+		} else {
 	    gnomecard_canvas_text_item_set(homephone, "");
-	}	    
-
-	l = findmatchPhoneType(crd->phone.l, PHONE_WORK);
-	if (l) {
+		}	    
+		
+		l = findmatchPhoneType(crd->phone.l, PHONE_WORK);
+		if (l) {
 	    CardPhone *phone;
 	    
 	    phone = (CardPhone *)l->data;
 	    gnomecard_canvas_text_item_set(workphone, phone->data);
-	} else {
+		} else {
 	    gnomecard_canvas_text_item_set(workphone, "");
-	}
-    } else {
-	gnomecard_canvas_text_item_set(homephone, "");
-    }	
-
-    /* address */
-    if (crd->postal.l) {
-	l = findmatchAddrType(crd->postal.l, ADDR_HOME);
-	if (l) {
+		}
+	} else {
+		gnomecard_canvas_text_item_set(homephone, "");
+		gnomecard_canvas_text_item_set(workphone, "");
+	}	
+	
+	/* address */
+	if (crd->postal.l) {
+		l = findmatchAddrType(crd->postal.l, ADDR_HOME);
+		if (l) {
 	    gchar tmp[200];
+			gchar *fmt1 = "%s, %s   %s";
+			gchar *fmt2 = "%s%s  %s";
+			gchar *fmt;
 
 	    CardPostAddr *p=(CardPostAddr *)l->data;
-
+			
+			if (p->city && *p->city && p->state && *p->state)
+				fmt = fmt1;
+			else
+				fmt = fmt2;
+			
 	    gnomecard_canvas_text_item_set(homeaddr.street1, 
-					   (p->street1) ? p->street1 : "");
+																		 (p->street1) ? p->street1 : "");
 	    gnomecard_canvas_text_item_set(homeaddr.street2,  
-					   (p->street2) ? p->street2 : "");
-
-	    snprintf(tmp, sizeof(tmp), "%s, %s   %s",
-		     (p->city) ? p->city : "", (p->state) ? p->state : "",
-		     (p->zip) ? p->zip : "");
+																		 (p->street2) ? p->street2 : "");
+			
+	    snprintf(tmp, sizeof(tmp), fmt, 
+							 (p->city) ? p->city : "", (p->state) ? p->state : "",
+							 (p->zip) ? p->zip : "");
 	    gnomecard_canvas_text_item_set(homeaddr.state, tmp);
 	    gnomecard_canvas_text_item_set(homeaddr.country, 
-					    (p->country) ? p->country : "");
-	} else {
+																		 (p->country) ? p->country : "");
+		} else {
 	    gnomecard_canvas_text_item_set(homeaddr.street1, "");
 	    gnomecard_canvas_text_item_set(homeaddr.street2, "");
 	    gnomecard_canvas_text_item_set(homeaddr.state, "");
 	    gnomecard_canvas_text_item_set(homeaddr.country, "");
-	}
-
-	l = findmatchAddrType(crd->postal.l, ADDR_WORK);
-	if (l) {
-	    gchar tmp[200];
+		}
+		
+		l = findmatchAddrType(crd->postal.l, ADDR_WORK);
+		if (l) {
+	    gchar tmp[200];	
+			gchar *fmt1 = "%s, %s   %s";
+			gchar *fmt2 = "%s%s  %s";
+			gchar *fmt;
 
 	    CardPostAddr *p=(CardPostAddr *)l->data;
-
+			
+			if (p->city && *p->city && p->state && *p->state)
+				fmt = fmt1;
+			else
+				fmt = fmt2;
+			
 	    gnomecard_canvas_text_item_set(workaddr.street1, 
-					   (p->street1) ? p->street1 : "");
+																		 (p->street1) ? p->street1 : "");
 	    gnomecard_canvas_text_item_set(workaddr.street2,  
-					   (p->street2) ? p->street2 : "");
-
-	    snprintf(tmp, sizeof(tmp), "%s, %s   %s",
-		     (p->city) ? p->city : "", (p->state) ? p->state : "",
-		     (p->zip) ? p->zip : "");
+																		 (p->street2) ? p->street2 : "");
+			
+	    snprintf(tmp, sizeof(tmp), fmt,
+							 (p->city) ? p->city : "", (p->state) ? p->state : "",
+							 (p->zip) ? p->zip : "");
 	    gnomecard_canvas_text_item_set(workaddr.state, tmp);
 	    gnomecard_canvas_text_item_set(workaddr.country, 
-					    (p->country) ? p->country : "");
-	} else {
+																		 (p->country) ? p->country : "");
+		} else {
 	    gnomecard_canvas_text_item_set(workaddr.street1, "");
 	    gnomecard_canvas_text_item_set(workaddr.street2, "");
 	    gnomecard_canvas_text_item_set(workaddr.state, "");
 	    gnomecard_canvas_text_item_set(workaddr.country, "");
+		}
+	} else {
+		gnomecard_canvas_text_item_set(homeaddr.street1, "");
+		gnomecard_canvas_text_item_set(homeaddr.street2, "");
+		gnomecard_canvas_text_item_set(homeaddr.state, "");
+		gnomecard_canvas_text_item_set(homeaddr.country, "");
+		gnomecard_canvas_text_item_set(workaddr.street1, "");
+		gnomecard_canvas_text_item_set(workaddr.street2, "");
+		gnomecard_canvas_text_item_set(workaddr.state, "");
+		gnomecard_canvas_text_item_set(workaddr.country, "");
 	}
-    } else {
-	gnomecard_canvas_text_item_set(homeaddr.street1, "");
-	gnomecard_canvas_text_item_set(homeaddr.street2, "");
-	gnomecard_canvas_text_item_set(homeaddr.state, "");
-	gnomecard_canvas_text_item_set(homeaddr.country, "");
-	gnomecard_canvas_text_item_set(workaddr.street1, "");
-	gnomecard_canvas_text_item_set(workaddr.street2, "");
-	gnomecard_canvas_text_item_set(workaddr.state, "");
-	gnomecard_canvas_text_item_set(workaddr.country, "");
-    }
-
-    /* memos */
-    if (crd->comment.str)
-	gnomecard_canvas_text_item_set(memos, crd->comment.str);
-    else
-	gnomecard_canvas_text_item_set(memos, "");
-
-    /* set scrolling region */
-    gnome_canvas_item_get_bounds(GNOME_CANVAS_ITEM(root), &x1,&y1,&x2,&y2);
-    gnome_canvas_set_scroll_region(GNOME_CANVAS(gnomecard_canvas),
-				   x1, y1, x2, y2);
-    
+	
+	/* memos */
+	if (crd->comment.str)
+		gnomecard_canvas_text_item_set(memos, crd->comment.str);
+	else
+		gnomecard_canvas_text_item_set(memos, "");
+	
+	/* set scrolling region */
+	gnome_canvas_item_get_bounds(GNOME_CANVAS_ITEM(root), &x1,&y1,&x2,&y2);
+	gnome_canvas_set_scroll_region(GNOME_CANVAS(gnomecard_canvas),
+																 x1, y1, x2, y2);
 }
